@@ -56,6 +56,8 @@
 #'
 #' @param .env An environment.
 #' @param ... Pairs of names and expressions, values or functions.
+#'   Pass [zap()] objects to remove bindings.
+#'
 #'   These dots support [tidy dots][tidy-dots] features.
 #' @return The input object `.env`, with its associated environment
 #'   modified in place, invisibly.
@@ -110,7 +112,7 @@ env_bind <- function(.env, ...) {
 }
 env_bind_impl <- function(env, data, fn, bind = FALSE, binder = NULL) {
   if (!is_vector(data)) {
-    type <- as_friendly_type(type_of(data))
+    type <- friendly_type_of(data)
     abort(sprintf("`data` must be a vector not a %s", type))
   }
   if (length(data)) {
@@ -123,7 +125,7 @@ env_bind_impl <- function(env, data, fn, bind = FALSE, binder = NULL) {
     }
   }
 
-  env_ <- get_env_retired(env, fn, caller_env(2))
+  env_ <- get_env_retired(env, fn)
   nms <- names2(data)
 
   if (bind) {
@@ -350,7 +352,7 @@ env_bury <- function(.env, ...) {
 #'   modified in place, invisibly.
 #' @export
 #' @examples
-#' data <- set_names(as_list(letters), letters)
+#' data <- set_names(as.list(letters), letters)
 #' env_bind(environment(), !!! data)
 #' env_has(environment(), letters)
 #'
@@ -553,16 +555,16 @@ scope_poke <- function(env, nm, value, create) {
 #' @section Encoding:
 #'
 #' There are deep encoding issues when you convert a string to symbol
-#' and vice versa. Symbols are _always_ in the native encoding (see
-#' [set_chr_encoding()]). If that encoding (let's say latin1) cannot
-#' support some characters, these characters are serialised to
-#' ASCII. That's why you sometimes see strings looking like
-#' `<U+1234>`, especially if you're running Windows (as R doesn't
-#' support UTF-8 as native encoding on that platform).
+#' and vice versa. Symbols are _always_ in the native encoding. If
+#' that encoding (let's say latin1) cannot support some characters,
+#' these characters are serialised to ASCII. That's why you sometimes
+#' see strings looking like `<U+1234>`, especially if you're running
+#' Windows (as R doesn't support UTF-8 as native encoding on that
+#' platform).
 #'
 #' To alleviate some of the encoding pain, `env_names()` always
 #' returns a UTF-8 character vector (which is fine even on Windows)
-#' with unicode points unserialised.
+#' with ASCII unicode points translated back to UTF-8.
 #'
 #' @inheritParams get_env
 #' @return A character vector of object names.
