@@ -3,7 +3,7 @@ context("call")
 # Creation ----------------------------------------------------------------
 
 test_that("character vector must be length 1", {
-  expect_error(call2(letters), "must be a length 1 string")
+  expect_error(call2(letters), "must be a string")
 })
 
 test_that("args can be specified individually or as list", {
@@ -25,6 +25,16 @@ test_that("succeeds with literal functions", {
   expect_error(regex = NA, call2(base::list, 1:10))
 })
 
+test_that("call2() preserves empty arguments", {
+  expect_identical(call2("[", quote(x), , drop = ), quote(x[, drop = ]))
+})
+
+test_that("call2() requires a symbol when namespace is supplied", {
+  expect_identical(call2("foo", .ns = "bar"), quote(bar::foo()))
+  expect_error(call2(function() NULL, .ns = "bar"), "must be a string or symbol")
+  expect_error(call2(quote(foo()), .ns = "bar"), "must be a string or symbol")
+})
+
 
 # Standardisation ---------------------------------------------------------
 
@@ -75,7 +85,7 @@ test_that("can modify calls for functions containing dots", {
 test_that("accepts unnamed arguments", {
   expect_identical(
     call_modify(~get(), "foo", envir = "bar", "baz"),
-    ~get(envir = "bar", "foo", "baz")
+    ~get("foo", envir = "bar", "baz")
   )
 })
 
@@ -111,6 +121,32 @@ test_that("can't supply unnamed zaps", {
   expect_error(call_modify(quote(foo(bar)), zap()), "can't be unnamed")
 })
 
+test_that("positions are not changed", {
+  expect_identical(call_modify(quote(fn(1)), x = "foo"), quote(fn(1, x = "foo")))
+  expect_identical(call_modify(quote(fn(x = 1)), x = "foo"), quote(fn(x = "foo")))
+  expect_identical(call_modify(quote(fn(1, x = 1)), x = "foo"), quote(fn(1, x = "foo")))
+  expect_identical(call_modify(quote(fn(x = 1, 1)), x = "foo"), quote(fn(x = "foo", 1)))
+
+  expect_identical(call_modify(quote(fn(1)), ... = ), quote(fn(1, ...)))
+  expect_identical(call_modify(quote(fn(...)), ... = ), quote(fn(...)))
+  expect_identical(call_modify(quote(fn(1, ...)), ... = ), quote(fn(1, ...)))
+  expect_identical(call_modify(quote(fn(..., 1)), ... = ), quote(fn(..., 1)))
+
+  expect_identical(call_modify(quote(fn()), 1, x = "foo"), quote(fn(1, x = "foo")))
+  expect_identical(call_modify(quote(fn()), x = 1, x = "foo"), quote(fn(x = "foo")))
+  expect_identical(call_modify(quote(fn()), 1, x = 1, x = "foo"), quote(fn(1, x = "foo")))
+  expect_identical(call_modify(quote(fn()), x = 1, 1, x = "foo"), quote(fn(x = "foo", 1)))
+
+  expect_identical(call_modify(quote(fn()), 1, ... = ), quote(fn(1, ...)))
+  expect_identical(call_modify(quote(fn()), ... = , ... = ), quote(fn(...)))
+  expect_identical(call_modify(quote(fn()), 1, ... = , ... = ), quote(fn(1, ...)))
+  expect_identical(call_modify(quote(fn()), ... = , 1, ... = ), quote(fn(..., 1)))
+})
+
+test_that("empty quosures are treated as empty args", {
+  expect_identical(call_modify(quote(fn()), ... = quo()), quote(fn(...)))
+})
+
 
 # Utils --------------------------------------------------------------
 
@@ -155,11 +191,8 @@ test_that("call_name() handles namespaced and anonymous calls", {
   expect_null(call_name(quote((function() NULL)())))
 })
 
-test_that("call_name() handles formulas and frames", {
+test_that("call_name() handles formulas", {
   expect_identical(call_name(~foo(baz)), "foo")
-
-  fn <- function() call_name(call_frame())
-  expect_identical(fn(), "fn")
 })
 
 test_that("call_fn() extracts function", {
@@ -419,3 +452,16 @@ test_that("call_print_fine_type() returns correct enum", {
   expect_identical(call_print_fine_type(quote(`$`(a, b, c))), "infix")
   expect_identical(call_print_fine_type(quote(`@`(a, b, c))), "infix")
 })
+
+test_that("call_name() fails with namespaced objects (#670)", {
+  skip("Disabled for the 0.3.1 release")
+  expect_error(call_name(~foo::bar), "`call` must be a quoted call")
+  expect_error(call_name(~foo:::bar), "`call` must be a quoted call")
+})
+
+test_that("call_ns() retrieves namespaces", {
+  expect_error(call_ns(quote(foo)), "must be a quoted call")
+  expect_null(call_ns(quote(foo())))
+  expect_identical(call_ns(quote(foo::bar())), "foo")
+  expect_identical(call_ns(quote(foo:::bar())), "foo")
+})
